@@ -26,10 +26,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <X11/Xlib.h>
-
-#ifdef HAVE_DBUS_1_1
 #include <dbus/dbus.h>
-#endif
 
 #include <stdarg.h>
 #include <string.h>
@@ -53,10 +50,7 @@
 /* From avahi-common/domain.h */
 #define AVAHI_LABEL_MAX 64
 
-#ifdef HAVE_DBUS_1_1
 static char *dbus_session_id;
-#endif
-
 static pid_t httpd_pid = 0;
 
 static int
@@ -151,7 +145,6 @@ get_share_name (void)
 	return name;
 }
 
-#ifdef HAVE_DBUS_1_1
 static void
 init_dbus() {
 	/* The only use we make of D-BUS is to fetch the session BUS ID so we can export
@@ -186,7 +179,6 @@ init_dbus() {
 	dbus_connection_close(connection);
 	dbus_connection_unref(connection);
 }
-#endif
 
 static void
 ensure_conf_dir (void)
@@ -466,10 +458,7 @@ http_down (void)
 gboolean
 http_init (void)
 {
-#ifdef HAVE_DBUS_1_1
 	init_dbus();
-#endif	
-
 	return TRUE;
 }
 
