@@ -26,6 +26,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <glib/gi18n-lib.h>
+#include <gio/gdesktopappinfo.h>
 #include <gtk/gtk.h>
 #include <bluetooth-client.h>
 #include <libnautilus-extension/nautilus-menu-provider.h>
@@ -52,43 +53,31 @@ static GType nautilus_user_share_get_type      (void);
 static void  nautilus_user_share_register_type (GTypeModule *module);
 
 static void
-launch_process (char **argv, GtkWindow *parent)
+launch_prefs_on_window (void)
 {
-        GError *error;
-        GtkWidget *dialog;
+	GDesktopAppInfo *app_info;
+	GError *error = NULL;
+	GdkAppLaunchContext *launch_context;
 
-        error = NULL;
-        if (!g_spawn_async (NULL,
-                            argv, NULL,
-                            0,
-                            NULL, NULL,
-                            NULL,
-                            &error)) {
+	app_info = g_desktop_app_info_new ("gnome-sharing-panel.desktop");
+	if (app_info == NULL) {
+		g_warning ("Could not launch Sharing settings: gnome-sharing-panel.desktop missing");
+		return;
+	}
 
+	/* setup the launch context so the startup notification is correct */
+	launch_context = gdk_display_get_app_launch_context (gdk_display_get_default ());
+	gdk_app_launch_context_set_timestamp (launch_context, GDK_CURRENT_TIME);
 
-                dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING,
-						 GTK_BUTTONS_OK, _("Unable to launch the Personal File Sharing Preferences"));
+	if (!g_app_info_launch (G_APP_INFO (app_info), NULL, G_APP_LAUNCH_CONTEXT (launch_context), &error)) {
+		g_warning ("Could not launch '%s': %s",
+			   g_app_info_get_commandline (G_APP_INFO (app_info)),
+			   error->message);
+		g_error_free (error);
+	}
 
-                gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", error->message);
-
-                gtk_dialog_run (GTK_DIALOG (dialog));
-                gtk_widget_destroy (dialog);
-
-                g_error_free (error);
-        }
-}
-
-static void
-launch_prefs_on_window (GtkWindow *window)
-{
-        char *argv [2];
-
-        argv [0] = g_build_filename (BINDIR, "gnome-file-share-properties", NULL);
-        argv [1] = NULL;
-
-        launch_process (argv, window);
-
-        g_free (argv [0]);
+	g_object_unref (launch_context);
+	g_object_unref (app_info);
 }
 
 static void
@@ -96,9 +85,8 @@ bar_response_cb (NautilusShareBar *bar,
                  gint response,
                  gpointer         data)
 {
-        if (response == NAUTILUS_SHARE_BAR_RESPONSE_PREFERENCES) {
-                launch_prefs_on_window (GTK_WINDOW (data));
-        }
+        if (response == NAUTILUS_SHARE_BAR_RESPONSE_PREFERENCES)
+                launch_prefs_on_window ();
 }
 
 static void
